<?php

/**
 * @file
 * Admin page callbacks for the compare role permissions module.
 */

/**
 * Implements hook_form().
 */
function compare_role_permissions_form(&$form_state) {
  $form = array();

  $form['role1'] = array(
    '#type' => 'select',
    '#title' => 'Role',
    '#required' => TRUE,
    '#options' => user_roles(),
    '#default_value' => isset($form_state['values']['role1']) ? $form_state['values']['role1'] : DRUPAL_ANONYMOUS_RID,
  );

  $form['role2'] = array(
    '#type' => 'select',
    '#title' => 'Role',
    '#required' => TRUE,
    '#options' => user_roles(),
    '#default_value' => isset($form_state['values']['role2']) ? $form_state['values']['role2'] : DRUPAL_AUTHENTICATED_RID,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Compare',
  );

  if (!empty($form_state['results_table'])) {
    $form['results_table'] = array(
        '#type' => 'item',
        '#title' => t('Row'),
        '#value' => $form_state['results_table'],
        '#prefix' => '',
        '#suffix' => '',
    );
  }

  return $form;
}

/**
 * Implements hook_form_validate().
 */
function compare_role_permissions_form_validate($form, &$form_state) {
  if ($form_state['values']['role1'] == $form_state['values']['role2']) {
    form_set_error('role2', t('Role should be different from first.'));
  }
}

/**
 * Implements hook_form_submit().
 */
function compare_role_permissions_form_submit($form, &$form_state) {
  $all_role = user_roles();
  $role1 = $form_state['values']['role1'];
  $role2 = $form_state['values']['role2'];
  $header = array(
    array('data' => t('Permission')),
    array('data' => $all_role[$role1]),
    array('data' => $all_role[$role2]),
  );

  $rows = array();
  $roleprm1 = compare_role_permissions_get_permission($role1);
  $roleprm2 = compare_role_permissions_get_permission($role2);

  $roleprm1_keys = array_keys($roleprm1);
  $roleprm1_keys = explode(',', str_replace(", ", "," , $roleprm1_keys[0]));

  $roleprm2_keys = array_keys($roleprm2);
  $roleprm2_keys = explode(',', str_replace(", ", ",", $roleprm2_keys[0]));

  $role1_but_not_role2 = array_diff($roleprm1_keys, $roleprm2_keys);
  $role2_but_not_role1 = array_diff($roleprm2_keys, $roleprm1_keys);
  $all_diff_perm = array_merge($role1_but_not_role2,$role2_but_not_role1 );

  if (!empty($all_diff_perm)) {
    for ($i = 0; $i < count($all_diff_perm); $i++) {
      $row = array();
      $row = array($all_diff_perm[$i], in_array($all_diff_perm[$i], $roleprm1_keys) ? 'Yes' : 'No', in_array($all_diff_perm[$i], $roleprm2_keys) ? 'Yes' : 'No');
      $rows[] = $row;
    }
    $table = theme('table', $header, $rows);
    $form_state['results_table'] = $table;
  }
  else {
    $form_state['results_table'] = 'Both the selected roles has similar permissions';
  }
  $form_state['rebuild'] = TRUE;
}

/**
 * Helper function to fetch permissions list that are enabled for the role id.
 *
 * @param int $rid
 *   Role id of role.
 *
 * @return array
 *   Module, permissions list of given role id.
 */
function compare_role_permissions_get_permission($rid) {
  $result = db_query("select rid, perm from {permission} where rid = %d", $rid);
  $permissions = array();
  while ($row = db_fetch_object($result)) {
    $permissions[$row->perm] = array($row->rid);
  }

  return $permissions;
}
